"use strict";
/*
 *  Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
 *  SPDX-License-Identifier: Apache-2.0
 */
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const client_cloudformation_1 = require("@aws-sdk/client-cloudformation");
const client_cognito_identity_provider_1 = require("@aws-sdk/client-cognito-identity-provider");
const client_dynamodb_1 = require("@aws-sdk/client-dynamodb");
const client_ec2_1 = require("@aws-sdk/client-ec2");
const client_ec2_instance_connect_1 = require("@aws-sdk/client-ec2-instance-connect");
const client_eventbridge_1 = require("@aws-sdk/client-eventbridge");
const client_iam_1 = require("@aws-sdk/client-iam");
const client_kms_1 = require("@aws-sdk/client-kms");
const client_lambda_1 = require("@aws-sdk/client-lambda");
const client_s3_1 = require("@aws-sdk/client-s3");
const client_s3_control_1 = require("@aws-sdk/client-s3-control");
const client_sagemaker_1 = require("@aws-sdk/client-sagemaker");
const client_service_catalog_1 = require("@aws-sdk/client-service-catalog");
const client_service_catalog_appregistry_1 = require("@aws-sdk/client-service-catalog-appregistry");
const client_service_quotas_1 = require("@aws-sdk/client-service-quotas");
const client_ssm_1 = require("@aws-sdk/client-ssm");
const client_sts_1 = require("@aws-sdk/client-sts");
const appRegistryService_1 = __importDefault(require("./helpers/appRegistryService"));
const cloudformationService_1 = __importDefault(require("./helpers/cloudformationService"));
const cognitoService_1 = __importDefault(require("./helpers/cognitoService"));
const dynamoDBService_1 = __importDefault(require("./helpers/dynamoDB/dynamoDBService"));
const s3Service_1 = __importDefault(require("./helpers/s3Service"));
const serviceCatalogService_1 = __importDefault(require("./helpers/serviceCatalogService"));
class AwsService {
    constructor(options) {
        const customBackoff = (retryCount) => {
            console.log(`retry count: ${retryCount}, waiting: 100ms`);
            return 100;
        };
        const { region, ddbTableName, userAgent } = options;
        const sdkOptions = Object.assign({ maxRetries: 5, retryDelayOptions: { customBackoff }, customUserAgent: userAgent || '' }, options);
        this.clients = {
            cloudformation: new client_cloudformation_1.CloudFormation(sdkOptions),
            cognito: new client_cognito_identity_provider_1.CognitoIdentityProvider(sdkOptions),
            ssm: new client_ssm_1.SSM(sdkOptions),
            ec2: new client_ec2_1.EC2(sdkOptions),
            ec2InstanceConnect: new client_ec2_instance_connect_1.EC2InstanceConnect(sdkOptions),
            eventBridge: new client_eventbridge_1.EventBridge(sdkOptions),
            serviceCatalog: new client_service_catalog_1.ServiceCatalog(sdkOptions),
            s3: new client_s3_1.S3(sdkOptions),
            sts: new client_sts_1.STS(sdkOptions),
            iam: new client_iam_1.IAM(sdkOptions),
            s3Control: new client_s3_control_1.S3Control(sdkOptions),
            ddb: new client_dynamodb_1.DynamoDB(sdkOptions),
            lambda: new client_lambda_1.Lambda(sdkOptions),
            sagemaker: new client_sagemaker_1.SageMaker(sdkOptions),
            kms: new client_kms_1.KMS(sdkOptions),
            appRegistry: new client_service_catalog_appregistry_1.ServiceCatalogAppRegistry(sdkOptions),
            serviceQuotas: new client_service_quotas_1.ServiceQuotas(sdkOptions)
        };
        this.helpers = {
            cloudformation: new cloudformationService_1.default(this.clients.cloudformation),
            s3: new s3Service_1.default(this.clients.s3),
            ddb: new dynamoDBService_1.default({ region, table: ddbTableName || '' }),
            serviceCatalog: new serviceCatalogService_1.default(this.clients.serviceCatalog),
            appRegistryService: new appRegistryService_1.default(this.clients.appRegistry, this.clients.cloudformation, this.clients.serviceQuotas),
            cognito: new cognitoService_1.default(this.clients.cognito)
        };
    }
    async getAwsServiceForRole(params) {
        const { Credentials } = await this.clients.sts.assumeRole({
            RoleArn: params.roleArn,
            RoleSessionName: params.roleSessionName,
            ExternalId: params.externalId
        });
        if (Credentials) {
            return new AwsService({
                region: params.region,
                credentials: {
                    accessKeyId: Credentials.AccessKeyId,
                    secretAccessKey: Credentials.SecretAccessKey,
                    sessionToken: Credentials.SessionToken
                }
            });
        }
        else {
            throw new Error(`Unable to assume role with params: ${params}`);
        }
    }
}
exports.default = AwsService;
//# sourceMappingURL=awsService.js.map