"use strict";
/*
 *  Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
 *  SPDX-License-Identifier: Apache-2.0
 */
Object.defineProperty(exports, "__esModule", { value: true });
class CloudformationService {
    constructor(cloudformation) {
        this._cloudformation = cloudformation;
    }
    /**
     * Get Cloudformation output from a stack
     * @example
     * ```
     * // returns { 'lambdaRoleArn': "ExampleRoleArn" }
     * getCfnOutput('stack123', ['lambdaRoleArn'])
     * ```
     *
     * @param stackName - Stack name of CFN stack to get cloudformation output from
     * @param outputKeys - Output keys to get value for
     * @returns An object where the key is the output key and the value is the cloudformation output of that key.
     */
    async getCfnOutput(stackName, outputKeys) {
        const outputValues = {};
        for (const outputKey of outputKeys) {
            const describeStackParam = {
                StackName: stackName
            };
            const stackOutput = await this._cloudformation.describeStacks(describeStackParam);
            const describeStackResponse = stackOutput.Stacks[0].Outputs.find((output) => {
                return output.OutputKey && output.OutputKey.includes(outputKey);
            });
            if (describeStackResponse && describeStackResponse.OutputValue) {
                // eslint-disable-next-line security/detect-object-injection
                outputValues[outputKey] = describeStackResponse.OutputValue;
            }
            else {
                throw new Error(`Cannot find output value for ${outputKey}`);
            }
        }
        return outputValues;
    }
}
exports.default = CloudformationService;
//# sourceMappingURL=cloudformationService.js.map