"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const util_dynamodb_1 = require("@aws-sdk/util-dynamodb");
const lodash_1 = __importDefault(require("lodash"));
const paginationHelper_1 = require("../../../utilities/paginationHelper");
const batchEdit_1 = __importDefault(require("./batchEdit"));
const ddbUtil_1 = require("./ddbUtil");
const deleter_1 = __importDefault(require("./deleter"));
const getter_1 = __importDefault(require("./getter"));
const query_1 = __importDefault(require("./query"));
const scanner_1 = __importDefault(require("./scanner"));
const transactEdit_1 = __importDefault(require("./transactEdit"));
const updater_1 = __importDefault(require("./updater"));
class DynamoDBService {
    constructor(constants) {
        const { region, table } = constants;
        this._awsRegion = region;
        this._tableName = table;
    }
    /**
     * Creates a Scanner to do scan operations on a Dynamo DB table.
     *
     * @param params - optional object of optional properties to generate a scan request
     * @returns A Scanner
     *
     * @example Only use this method to set up the Scanner in an external file
     * ```ts
     * const scanner = dynamoDBService.scan({index: 'some-index'});
     * const dataFromScanOnIndex = await scanner.execute();
     * ```
     *
     * @example Use this method to set up the Scanner and then manually edit params with scanner methods
     * ```ts
     * const scanner = dynamoDBService.scan().index({index: 'some-index'});
     * const dataFromScanOnIndex = await scanner.execute();
     * ```
     */
    scan(params) {
        let scanner = new scanner_1.default({ region: this._awsRegion }, this._tableName);
        if (params) {
            if (params.index) {
                scanner = scanner.index(params.index);
            }
            if (params.start) {
                scanner = scanner.start((0, util_dynamodb_1.marshall)(params.start));
            }
            if (params.filter) {
                scanner = scanner.filter(params.filter);
            }
            if (params.strong === true) {
                scanner = scanner.strong();
            }
            if (params.names) {
                scanner = scanner.names(params.names);
            }
            if (params.values) {
                scanner = scanner.values((0, util_dynamodb_1.marshall)(params.values));
            }
            if (params.projection) {
                scanner = scanner.projection(params.projection);
            }
            if (params.select) {
                scanner = scanner.select(params.select);
            }
            if (params.limit) {
                scanner = scanner.limit(params.limit);
            }
            if (!lodash_1.default.isUndefined(params.totalSegment) && !lodash_1.default.isUndefined(params.segment)) {
                scanner = scanner.totalSegment(params.totalSegment).segment(params.segment);
            }
            if (params.capacity) {
                scanner = scanner.capacity(params.capacity);
            }
        }
        return scanner;
    }
    /**
     * Gets a single item from the DynamoDB table.
     *
     * @param params - {@link GetItemParams} object of properties to generate a get item request
     * @returns Promise of a string, {@link JSONValue} paired object
     *
     * @example Use this to get a single item from the DynamoDb table.
     * ```ts
     * const result = dynamoDBService.getItem({key: 'pk'});
     * ```
     */
    async getItem(params) {
        const result = (await this.get(params.key, params.params).execute());
        return result.Item;
    }
    /**
     * Creates a Getter to do single get item or batch get item operations on a DynamoDB table.
     *
     * @param key - single object of key to get for single get item or list of objects of keys to get for batch get item
     * @param params - optional object of optional properties to generate a get item request
     * @returns A Getter
     *
     * @example Only use this method to set up the Getter in an external file
     * ```ts
     * const getter = dynamoDBService.get({'pk': {S: 'pk'}, 'sk': {S: 'sk'}}, {projection: 'valueIWant'});
     * const dataFromGetValueIWant = await getter.execute();
     * ```
     *
     * @example Use this method to set up the Getter and then manually edit params with getter methods
     * ```ts
     * const getter = dynamoDBService.get({'pk': {S: 'pk'}, 'sk': {S: 'sk'}});
     * const dataFromGetValueIWant = await getter.projection({projection: 'valueIWant'}).execute();
     * ```
     */
    get(key, params) {
        let getter = new getter_1.default({ region: this._awsRegion }, this._tableName, key);
        if (params) {
            if (params.strong === true) {
                getter = getter.strong();
            }
            if (params.names) {
                getter = getter.names(params.names);
            }
            if (params.projection) {
                getter = getter.projection(params.projection);
            }
            if (params.capacity) {
                getter = getter.capacity(params.capacity);
            }
        }
        return getter;
    }
    /**
     * retrieves items from DynamoDB table.
     *
     * @param keys - array of keys to retrieve
     * @param params - optional object of optional properties to generate a get item request
     * @returns Promise\<Record\<string,JSONValue\>\>
     *
     * @example Use this method to retrieve an item from ddb by Id
     * ```ts
     * const item = await dynamoDBService.getItems([{'pk': 'pk', 'sk': 'sk'}, {'pk': 'pk2', 'sk': 'sk2'}], {projection: 'valueIWant'});
     * ```
     */
    async getItems(keys, params) {
        if (keys.length > ddbUtil_1.MAX_GET_ITEMS_SIZE)
            throw new Error(`Cannot retrieve more than ${ddbUtil_1.MAX_GET_ITEMS_SIZE} items by request.`);
        const batchGetResult = (await this.get(keys, params).execute());
        return batchGetResult.Responses[this._tableName].map((item) => {
            return item;
        });
    }
    /**
     * Queries the DynamoDB table.
     *
     * @param params - optional object of optional properties to generate a query request
     * @returns Promise<PaginatedJsonResponse>
     *
     * @example Use this to get paginated items from the DynamoDb table.
     * ```ts
     * const result = dynamoDBService.getPaginatedItems({sortKey: 'value', eq: {N: '5'}});
     * ```
     */
    async getPaginatedItems(params) {
        const result = await this.query(params).execute();
        const retrievedItems = result.Items || [];
        const data = retrievedItems.map((item) => item);
        return {
            data,
            paginationToken: (0, paginationHelper_1.getPaginationToken)(result)
        };
    }
    /**
     * Creates a Query to do query operations on a DynamoDB table.
     *
     * @param params - optional object of optional properties to generate a query request
     * @returns A Query
     *
     * @example Only use this method to set up the Query in an external file
     * ```ts
     * const query = dynamoDBService.query({sortKey: 'value', eq: {N: '5'}});
     * const queryValueEq5 = await query.execute();
     * ```
     *
     * @example Use this method to set up the Query and then manually edit params with query methods
     * ```ts
     * const query = dynamoDBService.query();
     * const queryValueEq5 = await query.sortKey('value').eq({N: '5'}).execute();
     * ```
     */
    query(params) {
        let query = new query_1.default({ region: this._awsRegion }, this._tableName);
        if (params) {
            if (params.index) {
                query = query.index(params.index);
            }
            if (params.key) {
                query = query.key(params.key.name, (0, util_dynamodb_1.marshall)(params.key.value, { removeUndefinedValues: true }));
            }
            if (params.sortKey) {
                query = query.sortKey(params.sortKey);
                // check only one condition was supplied
                if ([
                    ...new Set([
                        params.eq,
                        params.lt,
                        params.lte,
                        params.gt,
                        params.gte,
                        params.between,
                        params.begins
                    ])
                ].length > 2) {
                    throw new Error('You cannot query on two conditions seperately for sortKey');
                }
                if (params.eq) {
                    query = query.eq(params.eq);
                }
                else if (params.lt) {
                    query = query.lt(params.lt);
                }
                else if (params.lte) {
                    query = query.lte(params.lte);
                }
                else if (params.gt) {
                    query = query.gt(params.gt);
                }
                else if (params.gte) {
                    query = query.gte(params.gte);
                }
                else if (params.between) {
                    query = query.between(params.between.value1, params.between.value2);
                }
                else if (params.begins) {
                    query = query.begins(params.begins);
                }
            }
            else if (params.eq ||
                params.lt ||
                params.lte ||
                params.gt ||
                params.gte ||
                params.between ||
                params.begins) {
                throw new Error('You cannot query on sortKey without providing a sortKey name');
            }
            if (params.start) {
                query = query.start((0, util_dynamodb_1.marshall)(params.start));
            }
            if (params.filter) {
                query = query.filter(params.filter);
            }
            if (params.strong === true) {
                query = query.strong();
            }
            if (params.names) {
                query = query.names(params.names);
            }
            if (params.values) {
                query = query.values((0, util_dynamodb_1.marshall)(params.values, { removeUndefinedValues: true }));
            }
            if (params.projection) {
                query = query.projection(params.projection);
            }
            if (params.select) {
                query = query.select(params.select);
            }
            if (params.limit) {
                query = query.limit(params.limit);
            }
            if (!lodash_1.default.isUndefined(params.forward)) {
                query = query.forward(params.forward);
            }
            if (params.capacity) {
                query = query.capacity(params.capacity);
            }
        }
        return query;
    }
    /**
     * Places a DDB Update call and formats the response.
     *
     * @param updateParams - {@link UpdateParams} object to pass to the update request
     * @returns a {@link UpdateUnmarshalledOutput} item where Attributes is unmarshalled
     */
    async updateExecuteAndFormat(updateParams) {
        const result = await this.update(updateParams).execute();
        const unmarshalledResult = {
            Attributes: result.Attributes ? (0, util_dynamodb_1.unmarshall)(result.Attributes) : undefined,
            ConsumedCapacity: result.ConsumedCapacity || undefined,
            ItemCollectionMetrics: result.ItemCollectionMetrics || undefined
        };
        return unmarshalledResult;
    }
    /**
     * Creates an Updater to do update item operations on a DynamoDB table.
     *
     * @param key - object of key to update
     * @param params - optional object of optional properties to generate an update request
     * @returns A Updater
     *
     * @example Only use this method to set up the Updater in an external file
     * ```ts
     * const updater = dynamoDBService.update({'pk': {S: 'pk'}, 'sk': {S: 'sk}}, {item: {'newAttribute': {S: 'newValue'}}});
     * const dataFromUpdate = await updater.execute();
     * ```
     *
     * @example Use this method to set up the Updater and then manually edit params with updater methods
     * ```ts
     * const updater = dynamoDBService.update({'pk': {S: 'pk'}, 'sk': {S: 'sk}});
     * const dataFromUpdate = await updater.item({'newAttribute': {S: 'newValue'}}).execute();
     * ```
     */
    update(updateParams) {
        let updater = new updater_1.default({ region: this._awsRegion }, this._tableName, (0, util_dynamodb_1.marshall)(updateParams.key));
        if (updateParams.params) {
            if (updateParams.params.disableCreatedAt) {
                updater = updater.disableCreatedAt();
            }
            if (updateParams.params.disableUpdatedAt) {
                updater = updater.disableUpdatedAt();
            }
            if (updateParams.params.item) {
                updater = updater.item((0, util_dynamodb_1.marshall)(updateParams.params.item, { removeUndefinedValues: true }));
            }
            if (updateParams.params.set) {
                updater = updater.set(updateParams.params.set);
            }
            if (updateParams.params.add) {
                updater = updater.add(updateParams.params.add);
            }
            if (updateParams.params.remove) {
                updater = updater.remove(updateParams.params.remove);
            }
            if (updateParams.params.delete) {
                updater = updater.delete(updateParams.params.delete);
            }
            if (updateParams.params.names) {
                updater = updater.names(updateParams.params.names);
            }
            if (updateParams.params.values) {
                updater = updater.values((0, util_dynamodb_1.marshall)(updateParams.params.values));
            }
            if (updateParams.params.return) {
                updater = updater.return(updateParams.params.return);
            }
            if (updateParams.params.metrics) {
                updater = updater.metrics(updateParams.params.metrics);
            }
            if (updateParams.params.capacity) {
                updater = updater.capacity(updateParams.params.capacity);
            }
        }
        return updater;
    }
    /**
     * Deletes a single item from the DynamoDB table.
     *
     * @param params - {@link DeleteItemParams} object of properties to generate a delete item request
     * @returns Promise of a string, {@link JSONValue} paired object
     *
     * @example Use this to delete a single item from the DynamoDb table.
     * ```ts
     * const result = dynamoDBService.deleteItem({key: 'pk'});
     * ```
     */
    async deleteItem(params) {
        const result = (await this.delete(params.key, params.params).execute());
        return result.Attributes;
    }
    /**
     * Creates a Deleter to do single delete item operations on a DynamoDB table.
     *
     * @param key - object of key to delete
     * @param params - optional object of optional properties to generate a delete request
     * @returns A Deleter
     *
     * @example Only use this method to set up the Deleter in an external file
     * ```ts
     * const deleter = dynamoDBService.delete({'pk': {S: 'pk'}, 'sk': {S: 'sk}}, {condition: 'attribute_not_exists(DONOTDELETE)'});
     * const dataFromCondDelete = await deleter.execute();
     * ```
     *
     * @example Use this method to set up the Deleter and then manually edit params with deleter methods
     * ```ts
     * const deleter = dynamoDBService.delete({'pk': {S: 'pk'}, 'sk': {S: 'sk}});
     * const dataFromCondDelete = await deleter.condition('attribute_not_exists(DONOTDELETE)').execute();
     * ```
     */
    delete(key, params) {
        let deleter = new deleter_1.default({ region: this._awsRegion }, this._tableName, (0, util_dynamodb_1.marshall)(key));
        if (params) {
            if (params.condition) {
                deleter = deleter.condition(params.condition);
            }
            if (params.names) {
                deleter = deleter.names(params.names);
            }
            if (params.values) {
                deleter = deleter.values((0, util_dynamodb_1.marshall)(params.values));
            }
            if (params.return) {
                deleter = deleter.return(params.return);
            }
            if (params.capacity) {
                deleter = deleter.capacity(params.capacity);
            }
            if (params.metrics) {
                deleter = deleter.metrics(params.metrics);
            }
        }
        return deleter;
    }
    /**
     * Creates a BatchEdit to do batch write or delete operations on a DynamoDB table.
     *
     * @param params - optional object of optional properties to generate a batch edit request
     * @returns A BatchEdit
     *
     * @example Only use this method to set up the BatchEdit in an external file
     * ```ts
     * const batchEdit = dynamoDBService.batchEdit({addDeleteRequest: {'pk': {S: 'pk'}, 'sk': {S: 'sk'}}});
     * const dataFromBatchEditSingleDelete = await batchEdit.execute();
     * ```
     *
     * @example Use this method to set up the BatchEdit and then manually edit params with batch edit methods
     * ```ts
     * const batchEdit = dynamoDBService.batchEdit();
     * const dataFromBatchEditSingleDelete = await batchEdit.addDeleteRequest({'pk': {S: 'pk'}, 'sk': {S: 'sk'}}).execute();
     * ```
     */
    batchEdit(params) {
        let batchEdit = new batchEdit_1.default({ region: this._awsRegion }, this._tableName);
        if (params) {
            if (params.addDeleteRequest) {
                batchEdit = batchEdit.addDeleteRequest((0, util_dynamodb_1.marshall)(params.addDeleteRequest));
            }
            if (params.addWriteRequest) {
                batchEdit = batchEdit.addWriteRequest((0, util_dynamodb_1.marshall)(params.addWriteRequest));
            }
            if (params.addDeleteRequests) {
                batchEdit = batchEdit.addDeleteRequests(params.addDeleteRequests.map((request) => (0, util_dynamodb_1.marshall)(request)));
            }
            if (params.addWriteRequests) {
                batchEdit = batchEdit.addWriteRequests(params.addWriteRequests.map((request) => (0, util_dynamodb_1.marshall)(request)));
            }
        }
        return batchEdit;
    }
    /**
     * Commits transactions to the table
     *
     * @param params - the items for the transaction
     */
    async commitTransaction(params) {
        await this.transactEdit(params).execute();
    }
    /**
     * @deprecated Use `commitTransaction` instead
     * @param params - the items for the transaction
     * @returns A TransactEdit object
     */
    transactEdit(params) {
        let transactEdit = new transactEdit_1.default({ region: this._awsRegion }, this._tableName);
        if (params === null || params === void 0 ? void 0 : params.addDeleteRequests) {
            transactEdit = transactEdit.addDeleteRequests(params.addDeleteRequests.map((request) => (0, util_dynamodb_1.marshall)(request)));
        }
        if (params === null || params === void 0 ? void 0 : params.addPutItems) {
            transactEdit = transactEdit.addPutItems(params.addPutItems.map((request) => (0, util_dynamodb_1.marshall)(request, { removeUndefinedValues: true })));
        }
        if (params === null || params === void 0 ? void 0 : params.addPutRequests) {
            transactEdit = transactEdit.addPutRequests(params.addPutRequests.map((request) => {
                return {
                    item: (0, util_dynamodb_1.marshall)(request.item, { removeUndefinedValues: true }),
                    conditionExpression: request.conditionExpression,
                    expressionAttributeNames: request.expressionAttributeNames,
                    expressionAttributeValues: request.expressionAttributeValues
                        ? (0, util_dynamodb_1.marshall)(request.expressionAttributeValues, { removeUndefinedValues: true })
                        : undefined
                };
            }));
        }
        return transactEdit;
    }
    /**
     * @returns the table name
     */
    getTableName() {
        return this._tableName;
    }
}
exports.default = DynamoDBService;
//# sourceMappingURL=dynamoDBService.js.map