"use strict";
/*
 *  Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
 *  SPDX-License-Identifier: Apache-2.0
 */
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const crypto_1 = __importDefault(require("crypto"));
const awsService_1 = __importDefault(require("./aws/awsService"));
class CognitoTokenService {
    constructor(awsRegion, secretsService) {
        this._aws = new awsService_1.default({ region: awsRegion });
        this._secretsService = secretsService;
    }
    async generateCognitoTokenWithCredentials(userPoolId, clientId, userName, password, accountType) {
        const clientSecret = await this._getClientSecret(userPoolId, clientId);
        const secretHash = crypto_1.default
            .createHmac('SHA256', clientSecret)
            .update(userName + clientId)
            .digest('base64');
        const authParameters = {
            USERNAME: userName,
            PASSWORD: password,
            SECRET_HASH: secretHash
        };
        if (accountType === 'ADMIN') {
            return this._getAdminToken(userPoolId, clientId, authParameters);
        }
        if (accountType === 'USER') {
            return this._getUserToken(clientId, authParameters);
        }
        throw new Error(`accountType (${accountType}) must be 'USER' or 'ADMIN'`);
    }
    async _getAdminToken(userPoolId, clientId, authParameters) {
        const response = await this._aws.clients.cognito.adminInitiateAuth({
            UserPoolId: userPoolId,
            ClientId: clientId,
            AuthFlow: 'ADMIN_NO_SRP_AUTH',
            AuthParameters: authParameters
        });
        return {
            accessToken: response.AuthenticationResult.AccessToken,
            refreshToken: response.AuthenticationResult.RefreshToken,
            idToken: response.AuthenticationResult.IdToken
        };
    }
    async _getUserToken(clientId, authParameters) {
        const response = await this._aws.clients.cognito.initiateAuth({
            AuthFlow: 'USER_PASSWORD_AUTH',
            AuthParameters: authParameters,
            ClientId: clientId
        });
        return {
            accessToken: response.AuthenticationResult.AccessToken,
            refreshToken: response.AuthenticationResult.RefreshToken,
            idToken: response.AuthenticationResult.IdToken
        };
    }
    async generateCognitoToken(params) {
        const { userPoolId, clientId, rootUserName, rootPassword, rootUserNameParamStorePath, rootPasswordParamStorePath } = params;
        let password = rootPassword || '';
        if (rootPasswordParamStorePath && rootPassword) {
            throw new Error('Both "rootPasswordParamStorePath" and "rootPassword" are defined. Please pass in only one of the two parameters.');
        }
        else if (rootPasswordParamStorePath === undefined && rootPassword === undefined) {
            throw new Error('Either "rootPasswordParamStorePath" or "rootPassword" should be defined');
        }
        else if (rootPasswordParamStorePath) {
            password = await this._secretsService.getSecret(rootPasswordParamStorePath);
        }
        let userName = rootUserName || '';
        if (rootUserNameParamStorePath && rootUserName) {
            throw new Error('Both "rootUserNameParamStorePath" and "rootUserName" are defined. Please pass in only one of the two parameters.');
        }
        else if (rootUserNameParamStorePath === undefined && rootUserName === undefined) {
            throw new Error('Either "rootUserNameParamStorePath" or "rootUserName" should be defined');
        }
        else if (rootUserNameParamStorePath) {
            userName = await this._secretsService.getSecret(rootUserNameParamStorePath);
        }
        return this.generateCognitoTokenWithCredentials(userPoolId, clientId, userName, password, 'ADMIN');
    }
    async _getClientSecret(userPoolId, clientId) {
        const response = await this._aws.clients.cognito.describeUserPoolClient({
            UserPoolId: userPoolId,
            ClientId: clientId
        });
        return response.UserPoolClient.ClientSecret;
    }
}
exports.default = CognitoTokenService;
//# sourceMappingURL=cognitoTokenService.js.map