"use strict";
/*
 *  Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
 *  SPDX-License-Identifier: Apache-2.0
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.MetadataService = void 0;
const paginationHelper_1 = require("../utilities/paginationHelper");
const validatorHelper_1 = require("../utilities/validatorHelper");
const MAX_TRANSACTION_SIZE = 50;
class MetadataService {
    constructor(ddbService) {
        this._ddbService = ddbService;
    }
    /************************************************************
     * Retrieves metadata relationships related to entity and metadata related to dependencies
     * @param mainEntityRequest - main Entity request object
     * @param dependencies - array of dependencies
     * @param mainMetadataMapper - function that parses main entity request and dependency entity to main entity metadata
     * @param dependencyMetadataMapper - function that parses main entity request and dependency entity to dependency metadata
     * @returns object containing main entity metadata and dependency metadata
     ************************************************************/
    getMetadataItems(mainEntityRequest, dependencies, mainMetadataMapper, dependencyMetadataMapper) {
        const mainEntityMetadata = [];
        const dependencyMetadata = [];
        dependencies.forEach((dependency) => {
            const mainMetadataItem = mainMetadataMapper(mainEntityRequest, dependency);
            mainEntityMetadata.push(mainMetadataItem);
            if (dependencyMetadataMapper) {
                const dependencyMetadataItem = dependencyMetadataMapper(mainEntityRequest, dependency);
                dependencyMetadata.push(dependencyMetadataItem);
            }
        });
        return {
            mainEntityMetadata,
            dependencyMetadata
        };
    }
    /**
     *
     * @param mainEntityResourceType - main entity resource type
     * @param mainEntityId - main entity id
     * @param dependencyResourceType - dependency resource type
     * @param parser - Zod parser for metadata object
     * @param queryParams - parameters to query metadata. Supported page size and pagination token.
     * @returns object containing list of metadata objects and continuation token.
     */
    async listDependentMetadata(mainEntityResourceType, mainEntityId, dependencyResourceType, parser, queryParams) {
        var _a;
        let params = {
            key: { name: 'pk', value: `${mainEntityResourceType}#${mainEntityId}` },
            sortKey: 'sk',
            begins: { S: `${dependencyResourceType}#` },
            limit: Math.min((_a = queryParams === null || queryParams === void 0 ? void 0 : queryParams.pageSize) !== null && _a !== void 0 ? _a : paginationHelper_1.DEFAULT_API_PAGE_SIZE, paginationHelper_1.MAX_API_PAGE_SIZE)
        };
        params = (0, paginationHelper_1.addPaginationToken)(queryParams === null || queryParams === void 0 ? void 0 : queryParams.paginationToken, params);
        const { data, paginationToken } = await this._ddbService.getPaginatedItems(params);
        const dependencyMetadata = [];
        data.forEach((item) => {
            dependencyMetadata.push((0, validatorHelper_1.validateAndParse)(parser, item));
        });
        return { data: dependencyMetadata, paginationToken };
    }
    async updateRelationship(mainEntityResourceType, mainEntity, dependencyResourceType, dependencies) {
        if (!dependencies.length) {
            return;
        }
        if (dependencies.length > MAX_TRANSACTION_SIZE) {
            throw new Error(`Cannot add more than ${MAX_TRANSACTION_SIZE} dependencies in single transaction.`);
        }
        const pk = `${mainEntityResourceType}#${mainEntity.id}`;
        const items = [];
        dependencies.forEach((dependency) => {
            var _a, _b;
            const currentDate = new Date().toISOString();
            const mainWithDependency = Object.assign({ pk, sk: `${dependencyResourceType}#${dependency.id}`, id: dependency.id, createdAt: currentDate, updatedAt: currentDate }, ((_a = dependency.data) !== null && _a !== void 0 ? _a : {}));
            const dependencyWithMain = Object.assign({ pk: `${dependencyResourceType}#${dependency.id}`, sk: pk, id: mainEntity.id, createdAt: currentDate, updatedAt: currentDate }, ((_b = mainEntity.data) !== null && _b !== void 0 ? _b : {}));
            items.push(mainWithDependency);
            items.push(dependencyWithMain);
        });
        await this._ddbService
            .transactEdit({
            addPutItems: items
        })
            .execute();
    }
    async deleteRelationships(mainEntityResourceType, mainEntityId, dependencyResourceType, dependentIds) {
        if (!dependentIds.length) {
            return;
        }
        if (dependentIds.length > MAX_TRANSACTION_SIZE) {
            throw new Error(`Cannot delete more than ${MAX_TRANSACTION_SIZE} dependencies in single transaction.`);
        }
        const pk = `${mainEntityResourceType}#${mainEntityId}`;
        const items = [];
        dependentIds.forEach((dependencyId) => {
            const mainWithDependency = {
                pk,
                sk: `${dependencyResourceType}#${dependencyId}`
            };
            const dependencyWithMain = {
                pk: `${dependencyResourceType}#${dependencyId}`,
                sk: pk
            };
            items.push(mainWithDependency);
            items.push(dependencyWithMain);
        });
        await this._ddbService
            .transactEdit({
            addDeleteRequests: items
        })
            .execute();
    }
    /**
     * Retrieves a single instance of dependency metadata by Ids
     * @param mainEntityResourceType - main entity resource type
     * @param mainEntityId - main entity id
     * @param dependencyResourceType - dependency resource type
     * @param dependencyId - dependency entity id
     * @param parser - Zod parser for metadata object
     * @returns object with metadata information
     */
    async getMetadataItem(mainEntityResourceType, mainEntityId, dependencyResourceType, dependencyId, parser) {
        const item = await this._ddbService.getItem({
            key: {
                pk: `${mainEntityResourceType}#${mainEntityId}`,
                sk: `${dependencyResourceType}#${dependencyId}`
            }
        });
        if (item === undefined) {
            return undefined;
        }
        const resource = parser.parse(item);
        return Promise.resolve(resource);
    }
}
exports.MetadataService = MetadataService;
//# sourceMappingURL=metadataService.js.map